"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SjsParserPlugin = void 0;
const utils_1 = require("@morjs/utils");
const compilerPlugins_1 = require("../compilerPlugins");
const constants_1 = require("../constants");
/**
 * 多端编译的 sjs 解析和转换
 * 这里仅提供通用的处理, 端的差异由编译插件来解决
 */
class SjsParserPlugin {
    constructor() {
        this.name = 'SjsParserPlugin';
    }
    apply(runner) {
        runner.hooks.entryBuilder.tap(this.name, (entryBuilder) => {
            this.entryBuilder = entryBuilder;
        });
        // 放在 beforeRun 有助于保证拿到的是最终用户配置
        runner.hooks.beforeRun.tap(this.name, () => {
            // 非 compile 命令 跳过
            if (runner.commandName !== constants_1.COMPILE_COMMAND_NAME)
                return;
            const { target, sourceType } = runner.userConfig;
            // 如果同源编译，不做处理
            if (target === sourceType)
                return;
            const composedPlugins = (0, compilerPlugins_1.getComposedCompilerPlugins)();
            runner.hooks.sjsParser.tap(this.name, (transformers, options) => {
                // 添加 commonjs => esm 转换
                // 不处理 目标为 CommonJS 的文件
                // 如果源码平台和目标平台 模块类型一致 则不处理
                if (composedPlugins.compileModuleKind[target] !==
                    utils_1.CompileModuleKind.CommonJS &&
                    composedPlugins.compileModuleKind[sourceType] !==
                        composedPlugins.compileModuleKind[target]) {
                    transformers.before.unshift((0, utils_1.cjsToEsmTransformer)());
                }
                // 修改引用路径
                if (options.fileInfo.content.includes('import') ||
                    options.fileInfo.content.includes('require')) {
                    this.alterImportOrRequirePath(transformers, options.fileInfo.path);
                }
                return transformers;
            });
        });
    }
    /**
     * 修改 require 或 import 的引用路径
     */
    alterImportOrRequirePath(transformers, filePath) {
        transformers.before.push((0, utils_1.tsTransformerFactory)((node, context) => {
            var _a;
            const factory = context.factory;
            if (utils_1.typescript.isImportDeclaration(node)) {
                /**
                 * import 引用替换为正确的路径
                 */
                if (node.moduleSpecifier &&
                    utils_1.typescript.isStringLiteral(node.moduleSpecifier)) {
                    const importPath = node.moduleSpecifier.text;
                    const realImportPath = this.entryBuilder.getRealReferencePath(filePath, importPath, true);
                    if (realImportPath) {
                        return factory.updateImportDeclaration(node, node.decorators, node.modifiers, node.importClause, factory.createStringLiteral(realImportPath));
                    }
                }
            }
            else if (utils_1.typescript.isCallExpression(node) &&
                utils_1.typescript.isIdentifier(node.expression) &&
                node.expression.escapedText === 'require') {
                if (!((_a = node.arguments) === null || _a === void 0 ? void 0 : _a.length))
                    return node;
                const arg = node.arguments[0];
                if (!utils_1.typescript.isStringLiteral(arg))
                    return node;
                let importPath = arg.getText();
                if (!importPath)
                    return node;
                // 修正链接
                importPath = importPath.replace(/^('|")/, '').replace(/('|")$/, '');
                const realImportPath = this.entryBuilder.getRealReferencePath(filePath, importPath, true);
                if (realImportPath) {
                    return factory.updateCallExpression(node, node.expression, node.typeArguments, [factory.createStringLiteral(realImportPath)]);
                }
            }
            return node;
        }));
    }
}
exports.SjsParserPlugin = SjsParserPlugin;
//# sourceMappingURL=sjsParserPlugin.js.map